# -*- coding: utf-8 -*-


from bkr.client.task_watcher import *
from bkr.client import BeakerCommand, BeakerWorkflow, BeakerJob, BeakerRecipeSet, BeakerRecipe
from optparse import OptionValueError
import sys
import xml.dom.minidom

class Workflow_AutoFS(BeakerWorkflow):
    """AutoFS workflow to generate job to scheduler"""
    enabled = True
    doc = xml.dom.minidom.Document()

    def options(self):
        super(Workflow_AutoFS, self).options()
        self.parser.remove_option("--clients")
        # remove default --servers option and re-add with default of 3 servers
        self.parser.remove_option("--servers")
        self.parser.add_option(
            "--servers",
            default=3,
            type=int,
            help="Specify how many server hosts to be involved in multihost test",
        )
        self.parser.add_option(
            "--subtask",
            default=None,
            help="Only run the specified subtask",
        )
        self.parser.usage = "%%prog %s [options]" % self.normalized_name

    def run(self, *args, **kwargs):
        username = kwargs.get("username", None)
        password = kwargs.get("password", None)

        debug  = kwargs.get("debug", False)
        dryrun = kwargs.get("dryrun", False)
        wait = kwargs.get("wait", False)
	family = kwargs.get("family", None)
	distro = kwargs.get("distro", None)
	arches = kwargs.get("arches", [])
	taskParams = kwargs.get("taskparam", [])
        servers = kwargs.get("servers", 3)
        subtask = kwargs.get("subtask", None)

        # get all tasks requested
        requestedTasks = self.getTasks(*args, **kwargs)
 
        if not family and not distro:
            sys.stderr.write("No Family or Distro specified\n")
            sys.exit(1)

        if not arches:
            sys.stderr.write("No arches specified, you must specify at least one\n")
            sys.exit(1)

        if not requestedTasks:
            requestedTasks.append(dict(name="/CoreOS/autofs/bugzillas",
                                       arches=[]))

        if subtask:
            taskParams.append("RUNTEST=%s" % subtask)
            whiteboard = kwargs.pop("whiteboard")
            kwargs["whiteboard"] = "%s subtask:%s" % (whiteboard, subtask)

        # Create Job
        job = BeakerJob(*args, **kwargs)

        # Create Base Recipe
        recipeTemplate = BeakerRecipe()

        # Add Distro Requirements
        recipeTemplate.addBaseRequires(*args, **kwargs)

        # Add Host Requirements


        i386_node = self.doc.createElement('distro_arch')
        i386_node.setAttribute('op', '=')
        i386_node.setAttribute('value', 'i386')
        for arch in arches:
            arch_node = self.doc.createElement('distro_arch')
            arch_node.setAttribute('op', '=')
            arch_node.setAttribute('value', arch)
            if self.multi_host:
                recipeSet = BeakerRecipeSet(*args, **kwargs)
                recipeSet.addRecipe(self.processTemplate(recipeTemplate,
                                                   requestedTasks,
                                                   taskParams=taskParams,
                                                   distroRequires=arch_node,
                                                   role='CLIENTS',
                                                   arch=arch,
                                                   **kwargs))
                for i in range(self.n_servers):
                    recipeSet.addRecipe(self.processTemplate(recipeTemplate, 
                                                             requestedTasks,
                                                             taskParams=taskParams,
                                                             distroRequires=i386_node, 
                                                             role='SERVERS',
                                                             arch='i386',
                                                             **kwargs))
                job.addRecipeSet(recipeSet)
            else:
                job.addRecipe(self.processTemplate(recipeTemplate,
                                                   requestedTasks,
                                                   taskParams=taskParams,
                                                   distroRequires=arch_node,
                                                   arch=arch,
                                                   **kwargs))

        # jobxml
        jobxml = job.toxml(**kwargs)

        if debug:
            print jobxml

        self.set_hub(username, password)
        submitted_jobs = []
        failed = False

        if not dryrun:
            try:
                submitted_jobs.append(self.hub.jobs.upload(jobxml))
            except Exception, ex:
                failed = True
                print ex
        if not dryrun:
            print "Submitted: %s" % submitted_jobs
            if wait:
                watch_tasks(self.hub, submitted_jobs)
            if failed:
                sys.exit(1)
