# -*- coding: utf-8 -*-

import copy
from bkr.client.task_watcher import *
from bkr.client import BeakerCommand, BeakerWorkflow, BeakerJob, BeakerRecipeSet, BeakerRecipe, BeakerGuestRecipe
from optparse import OptionValueError
from random import randint
import fileinput
import sys
import os
import re
import xml.dom.minidom

def splitNVR(nvr):
    start=nvr
    retval = {}

    p3 = start.rfind(".")
    p2 = start.rfind("-",0,p3)
    p1 = start.rfind("-",0,p2)
    retval['tag'] = start[p1+1:]
    retval['release'] = start[p2+1:]
    retval['version'] = start[p1+1:p2]
    retval['name'] = start[:p1]
    return retval

class Matrixes(object):
    class RedHatEnterpriseLinux3(object):
        variant = 'AS'
        lite = {}
        lite['i386']     = [{'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                           ]
        lite['x86_64']   = [{'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                           ]
        lite['ia64']     = [{'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                           ]
        lite['ppc64']    = [{'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                           ]
        lite['s390']     = [{'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                           ]
        lite['s390x']    = [{'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                           ]
        lite['ppc']      = []

        matrix = {}
        matrix['i386']   = [{'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="!=" value="x86_64"/>
                                           <cpu_count op="=" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="!=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="!=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'hugemem','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="!=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'smp','gprop' : False},
                            {'sprop': """<hostRequires> 
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'hugemem','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'smp','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'smp','gprop' : False},
                           ];
        
        matrix['x86_64'] = [{'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires> 
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'smp','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'smp','gprop' : False},
                           ]
        matrix['ia64']   = [{'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                           ]
        matrix['s390']   = [{'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                           ]
        matrix['s390x']  = [{'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                           ]
        matrix['ppc64']  = [{'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires/>""", 
                             'ktype' : 'up','gprop' : False},
                           ]
        matrix['ppc']    = []

    class RedHatEnterpriseLinux4(object):
        variant = 'AS'
        lite = {}
        lite['i386']     = [{'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                           ]
        lite['x86_64']   = [{'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                           ]
        lite['ia64']     = [{'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                           ]
        lite['ppc64']    = [{'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                           ]
        lite['s390']     = [{'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                           ]
        lite['s390x']    = [{'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                           ]
        lite['ppc']      = []

        matrix = {}
        matrix['i386']   = [{'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="!=" value="x86_64"/>
                                           <cpu_count op="=" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="!=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="!=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'hugemem','gprop' : False},
                            {'sprop': """<hostRequires> 
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="!=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'smp','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'hugemem','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'smp','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'hugemem','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'smp','gprop' : False},
                           ]
        matrix['x86_64'] = [{'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'largesmp','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'smp','gprop' : False},
                            {'sprop': """<hostRequires> 
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="=" value="i386"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'largesmp','gprop' : False},
                            {'sprop': """<hostRequires> 
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="=" value="i386"/>
                                           <cpu_count op="&gt;" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'smp','gprop' : False},
                           ]
        matrix['ia64']   = [{'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires/>""",
                             'ktype' : 'largesmp','gprop' : False},
                            {'sprop': """<hostRequires/>""",
                             'ktype' : 'largesmp','gprop' : False},
                           ]
        matrix['s390']   = [{'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                           ]
        matrix['s390x']  = [{'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                           ]
        matrix['ppc64']  = [{'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires/>""",
                             'ktype' : 'largesmp','gprop' : False},
                            {'sprop': """<hostRequires/>""",
                             'ktype' : 'largesmp','gprop' : False},
                           ]
        matrix['ppc']    = []

    class RedHatEnterpriseLinuxServer5(object):
        variant = ''
        lite = {}
        lite['i386']     = [{'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                           ]
        lite['x86_64']   = [{'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                           ]
        lite['ia64']     = [{'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                           ]
        lite['ppc64']    = [{'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                           ]
        lite['s390x']    = [{'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                           ]

        kvm = {}
        kvm['x86_64']    = [{'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="=" value="x86_64"/>
                                           <key_value key="HVM" op="=" value="1"/>
                                           <key_value key="DISKSPACE" op="&gt;=" value="75000"/>
                                           <memory op="&gt;=" value="4096"/>
                                           <cpu_count op="&gt;=" value="4"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : True},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <key_value key="HVM" op="=" value="1"/>
                                           <key_value key="DISKSPACE" op="&gt;=" value="75000"/>
                                           <memory op="&gt;=" value="4096"/>
                                           <cpu_count op="&gt;=" value="4"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : True},
                           ]
        xen = {}
        xen['i386']      = [{'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="=" value="i386"/>
                                           <key_value key="HVM" op="=" value="1"/>
                                           <key_value key="CPUFLAGS" op="=" value="pae"/>
                                           <key_value key="DISKSPACE" op="&gt;=" value="75000"/>
                                           <memory op="&gt;=" value="4096"/>
                                           <cpu_count op="&gt;=" value="4"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'xen','gprop' : True},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="i386"/>
                                           <key_value key="HVM" op="=" value="1"/>
                                           <key_value key="DISKSPACE" op="&gt;=" value="75000"/>
                                           <memory op="&gt;=" value="4096"/>
                                           <cpu_count op="&gt;=" value="4"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'xen','gprop' : True},
                           ]

        xen_intel        = {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="=" value="x86_64"/>
                                           <key_value key="HVM" op="=" value="1"/>
                                           <key_value key="CPUFLAGS" op="=" value="pae"/>
                                           <key_value key="DISKSPACE" op="&gt;=" value="75000"/>
                                           <memory op="&gt;=" value="4096"/>
                                           <cpu_count op="&gt;=" value="4"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'xen','gprop' : True}
        xen_amd          = {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <key_value key="HVM" op="=" value="1"/>
                                           <key_value key="DISKSPACE" op="&gt;=" value="75000"/>
                                           <memory op="&gt;=" value="4096"/>
                                           <cpu_count op="&gt;=" value="4"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'xen','gprop' : True}

        xen_intel_i386_guests = copy.deepcopy(xen_intel)
        xen_amd_i386_guests = copy.deepcopy(xen_amd)
        xen_intel_i386_guests['garch'] = 'i386'
        xen_amd_i386_guests['garch'] = 'i386'

        xen['x86_64']    = [ xen_intel, xen_intel_i386_guests, xen_amd, xen_amd_i386_guests ]

        xen['ia64']      = [{'sprop': """<hostRequires>
                                           <key_value key="VENDOR" op="!=" value="SGI"/>
                                           <arch op="=" value="ia64"/>
                                           <key_value key="HVM" op="=" value="1"/>
                                           <key_value key="DISKSPACE" op="&gt;=" value="75000"/>
                                           <memory op="&gt;=" value="4096"/>
                                           <cpu_count op="&gt;=" value="4"/>
                                         </hostRequires>""",
                             'ktype' : 'xen','gprop' : True},
                            ]

        virt = {}
        virt['i386']     = []
        virt['x86_64']   = []
        virt['ia64']     = []
        virt['i386'].extend(xen['i386'])
        virt['x86_64'].extend(xen['x86_64'])
        virt['ia64'].extend(xen['ia64'])
        virt['x86_64'].extend(kvm['x86_64'])

        matrix = {}
        matrix['i386']   = [{'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="!=" value="x86_64"/>
                                           <cpu_count op="=" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires> 
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="!=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="!=" value="x86_64"/>
                                           <key_value key="CPUFLAGS" op="=" value="pae"/>
                                           <cpu_count op="=" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'PAE','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="!=" value="x86_64"/>
                                           <key_value key="CPUFLAGS" op="=" value="pae"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'PAE','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <key_value key="CPUFLAGS" op="=" value="pae"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'xen','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="=" value="x86_64"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires> 
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="=" value="x86_64"/>
                                           <key_value key="CPUFLAGS" op="=" value="pae"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'PAE','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <key_value key="CPUFLAGS" op="=" value="pae"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'PAE','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <key_value key="CPUFLAGS" op="=" value="pae"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'xen','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'debug','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'debug','gprop' : False},
                           ]
        matrix['i386'].extend(xen['i386'])
        matrix['x86_64'] = [{'sprop': """<hostRequires> 
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <cpu_count op="=" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires> 
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <key_value key="HVM" op="=" value="1"/>
                                           <cpu_count op="&gt;=" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'xen','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'debug','gprop' : False},
                            {'sprop': """<hostRequires> 
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <cpu_count op="=" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires> 
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <key_value key="HVM" op="=" value="1"/>
                                           <cpu_count op="&gt;=" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'xen','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'debug','gprop' : False},
                           ]
        matrix['x86_64'].extend(xen['x86_64'])
        matrix['x86_64'].extend(kvm['x86_64'])
        matrix['ia64']   = [{'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="VENDOR" op="!=" value="SGI"/>
                                           <key_value key="HVM" op="=" value="1"/>
                                         </hostRequires>""",
                             'ktype' : 'xen','gprop' : False},
                            {'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires/>""",
                             'ktype' : 'debug','gprop' : False},
                           ]
        matrix['ia64'].extend(xen['ia64'])
        matrix['s390x']  = [{'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires/>""",
                             'ktype' : 'debug','gprop' : False},
                           ]
        matrix['ppc64']  = [{'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires/>""",
                             'ktype' : 'debug','gprop' : False},
                           ]

        rt = {}
        rt['i386']       = [{'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'rt','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'rt','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'rt','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'rt','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'rt','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'rt-debug','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'rt-debug','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'rt-trace','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'rt-trace','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'rt-vanilla','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'rt-vanilla','gprop' : False},
                           ]
        rt['x86_64']     = [{'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'rt','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'rt','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <cpu_count op="&gt;" value="3"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'rt','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'rt','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'rt','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'rt-debug','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'rt-debug','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'rt-trace','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'rt-trace','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'rt-vanilla','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'rt-vanilla','gprop' : False},
                           ]

    class RedHatEnterpriseLinux6(object):
        variant = 'Server'
        lite = {}
        lite['i386']     = [{'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                           ]
        lite['x86_64']   = [{'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                           ]
        lite['ppc64']    = [{'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                           ]
        lite['s390x']    = [{'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                           ]

        kvm = {}
        kvm['x86_64']    = [{'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="=" value="x86_64"/>
                                           <key_value key="HVM" op="=" value="1"/>
                                           <key_value key="DISKSPACE" op="&gt;=" value="75000"/>
                                           <memory op="&gt;=" value="4096"/>
                                           <cpu_count op="&gt;=" value="4"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : True},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <key_value key="HVM" op="=" value="1"/>
                                           <key_value key="DISKSPACE" op="&gt;=" value="75000"/>
                                           <memory op="&gt;=" value="4096"/>
                                           <cpu_count op="&gt;=" value="4"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : True},
                           ]

        virt = {}
        virt['x86_64']   = []
        virt['x86_64'].extend(kvm['x86_64'])

        matrix = {}
        matrix['i386']   = [{'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="!=" value="x86_64"/>
                                           <key_value key="CPUFLAGS" op="=" value="pae"/>
                                           <cpu_count op="=" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="!=" value="x86_64"/>
                                           <key_value key="CPUFLAGS" op="=" value="pae"/>
                                           <cpu_count op="=" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="=" value="x86_64"/>
                                           <key_value key="CPUFLAGS" op="=" value="pae"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <key_value key="CPUFLAGS" op="=" value="pae"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},                     
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <key_value key="CPUFLAGS" op="=" value="pae"/>
                                           <memory op="&gt;=" value="12288"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <key_value key="CPUFLAGS" op="=" value="pae"/>
                                           <memory op="&gt;=" value="12288"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <key_value key="CPUFLAGS" op="=" value="pae"/>
                                           <memory op="&gt;=" value="4096"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <key_value key="CPUFLAGS" op="=" value="pae"/>
                                           <memory op="&gt;=" value="4096"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <key_value key="CPUFLAGS" op="=" value="pae"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'debug','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <key_value key="CPUFLAGS" op="=" value="pae"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'debug','gprop' : False},
                           ]
        matrix['x86_64'] = [{'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="=" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},                     
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'debug','gprop' : False},
                            {'sprop': """<hostRequires> 
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'debug','gprop' : False},
                           ]
        matrix['x86_64'].extend(kvm['x86_64'])
        matrix['ppc64']  = [{'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires/>""",
                             'ktype' : 'debug','gprop' : False},
                           ]
        matrix['s390x']  = [{'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires/>""",
                             'ktype' : 'debug','gprop' : False},
                           ]

    class RedHatEnterpriseLinux7(object):
        variant = 'Server'
        lite = {}
        lite['x86_64']   = [{'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False},
                           ]
        lite['ppc64']    = [{'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                           ]
        lite['s390x']    = [{'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                           ]

        kvm = {}
        kvm['x86_64']    = [{'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="=" value="x86_64"/>
                                           <key_value key="HVM" op="=" value="1"/>
                                           <key_value key="DISKSPACE" op="&gt;=" value="75000"/>
                                           <memory op="&gt;=" value="4096"/>
                                           <cpu_count op="&gt;=" value="4"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : True,
                             'garch' : 'x86_64'},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <key_value key="HVM" op="=" value="1"/>
                                           <key_value key="DISKSPACE" op="&gt;=" value="75000"/>
                                           <memory op="&gt;=" value="4096"/>
                                           <cpu_count op="&gt;=" value="4"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : True,
                             'garch' : 'x86_64'},
                           ]

        virt = {}
        virt['x86_64']   = []
        virt['x86_64'].extend(kvm['x86_64'])

        matrix = {}
        x86_64_fips       = {'sprop': """<hostRequires>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                                         'ktype' : 'up', 'gprop' : False, 'fips' : True}

        x86_64_intel      = {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False}
        x86_64_amd        = {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'up','gprop' : False}

        matrix['x86_64'] = [x86_64_intel,
                            x86_64_intel,
                            x86_64_amd,
                            x86_64_amd,
                            x86_64_fips,
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="GenuineIntel"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'debug','gprop' : False},
                            {'sprop': """<hostRequires>
                                           <key_value key="CPUVENDOR" op="=" value="AuthenticAMD"/>
                                           <arch op="=" value="x86_64"/>
                                           <cpu_count op="&gt;" value="1"/>
                                           <hypervisor op="=" value=""/>
                                         </hostRequires>""",
                             'ktype' : 'debug','gprop' : False}
                           ]
        matrix['x86_64'].extend(kvm['x86_64'])
        matrix['ppc64']  = [{'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires/>""",
                             'ktype' : 'debug','gprop' : False},
                           ]
        matrix['s390x']  = [{'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires/>""",
                             'ktype' : 'up','gprop' : False},
                            {'sprop': """<hostRequires/>""",
                             'ktype' : 'debug','gprop' : False},
                           ]

# Set up some global variables to be use in recipe(s)
rDistInstall='/distribution/install'
rVirtInstall='/distribution/virt/install'
rVirtStart='/distribution/virt/start'
rKernInstall='/distribution/kernelinstall'


class Workflow_Kernel(BeakerWorkflow):
    """Kernel workflow to generate job to scheduler"""
    enabled = True
    doc = xml.dom.minidom.Document()

    def options(self):
        super(Workflow_Kernel, self).options()

        # Remove uneeded inherited parser options
        self.parser.remove_option("--servers")
        self.parser.remove_option("--clients")
        self.parser.remove_option("--type")
        self.parser.remove_option("--systype")
        self.parser.remove_option("--machine")
        self.parser.remove_option("--package")

        self.parser.add_option(
            "--nvr",
            default=None,
            help="Specify the kernel(Name-Version-Release) to be used in recipe"
            )

        self.parser.add_option(
            "--kvm",
            default=False,
            action="store_true",
            help="Enable KVM guests in a recipe"
            )

        self.parser.add_option(
            "--addon",
            default=False,
            action="store_true",
            help="Enable specific additional tests in random recipes"
            )

        self.parser.add_option(
            "--lite",
            default=False,
            action="store_true",
            help="Enable smaller matrix for the recipe"
            )

        self.parser.add_option(
            "--virt",
            default=False,
            action="store_true",
            help="Virt ONLY guests in a recipe"
            )

        self.parser.add_option(
            "--type",
            default=None,
            help="Specify the test type to be used in recipe"
            )        

        self.parser.add_option(
            "--taskfile",
            dest="tasklist",
            default=None,
            help="Include these TEST(s) in test selection from file"
            )

        self.parser.add_option(
            "--kcov",
            dest="kcov",
            default=False,
            action="store_true",
            help="Enable kcov for coverage data collection"
            )

        self.parser.add_option(
            "--kdir",
            dest="kdir",
            default=None,
            help="Capture kernel coverage data only from this subdirectory"
            )

        self.parser.add_option(
            "--R4U9Update",
            dest="r4u9update",
            default=False,
            action="store_true",
            help="Automatically add in the systemUpdate test. Updating to RHEL4-U9"
            )

        self.parser.add_option(
            "--without-fips",
            dest="without_fips",
            default=False,
            action="store_true",
            help="Do not add recipes with fips=1, by default fips=1 recipes are included"
            )

        self.parser.usage = "%%prog %s [options]" % self.normalized_name

    def addTasks(self, recipe, taskParams, arch):
        """ Adding specific tasks """
        for rTask in self.rTasks:
            if arch not in rTask['arches']:
                recipe.addTask(rTask['name'], taskParams=taskParams)

    def addTasksFromFile(self, *args, **kwargs):
        # Get list of test(s) from a file
        tasklist = kwargs.get('tasklist')
        tasks = []
        if tasklist:
            if os.path.exists(tasklist):
                for line in fileinput.input(tasklist):
                    # If the line does not start with /, assume it is not a
                    # valid test and don't submit it to the scheduler.
                    if (re.match("^/", line)):
                        tasks.append(dict(name=line.rstrip(), arches=[]))
            else:
                print ("Unable to open file: %s" % (tasklist))
                print "Unable to continue!"
                sys.exit(15)
        return tasks

    def run(self, *args, **kwargs):
        username = kwargs.get("username", None)
        password = kwargs.get("password", None)

        debug  = kwargs.get("debug", False)
        dryrun = kwargs.get("dryrun", False)
        family = kwargs.get("family", None)
        distro = kwargs.get("distro", None)
        arches = kwargs.get("arches", [])
        taskParams = kwargs.get("taskparam", [])
        rVersion = kwargs.get("version", None)
        self.rNVR = kwargs.get("nvr", None)
        type = kwargs.get("type", None)
        rKVM = kwargs.get("kvm", False)
        rAddOn = kwargs.get("addon", False)
        rLite = kwargs.get("lite", False)
        without_fips = kwargs.get("without_fips", False)
        rVirt = kwargs.get("virt", False)
        rKcov = kwargs.get("kcov", False)
        kdir = kwargs.get("kdir", None)
        r4u9 = kwargs.get("r4u9update", None)
        ndump = kwargs.get("ndump", True)
        kdump = kwargs.get("kdump", True)
        rpkginstall = kwargs.get("install", [])

        if not family and not distro:
            family = self.getFamily(*args, **kwargs)

        if family:
            family = re.sub('[._][0-9]-Z-.*$', '', family)
            if (family == 'RHEL-5'):
                family = re.sub('RHEL[-]?', 'RedHatEnterpriseLinuxServer',
                                (family))
            else:
                family = re.sub('RHEL[-]?', 'RedHatEnterpriseLinux',
                                (family))
                family = re.sub('FC[-]?', 'FedoraCore', (family))

            kwargs['family'] = family

        self.rTasks = self.getTasks(*args, **kwargs)
        self.rTasks.extend(self.addTasksFromFile(*args, **kwargs))

        # run Crasher tasks as last ones
        # self.rTasks is list, remove Crasher tasks and re-append to the end
        crashers_kwargs = copy.deepcopy(kwargs)
        crashers_kwargs["type"] = "Crasher"
        crasher_tasks = self.getTasks(*args, **crashers_kwargs)
        for crasher_task in crasher_tasks:
            if crasher_task in self.rTasks:
                self.rTasks.remove(crasher_task)
                self.rTasks.append(crasher_task)

        if not kwargs['variant']:
            kwargs['variant'] = getattr(Matrixes, self.getFamily(**kwargs), None).variant

        if not arches:
            arches = self.getArches(*args, **kwargs)

        # Split nvr into Name Version Release
        if self.rNVR:
            Nameit = splitNVR(self.rNVR)
            kName = (Nameit['name'])
            kVersion = (Nameit['version'])
            kRelease = (Nameit['release'])
            kVR = (Nameit['tag'])
        else:
            kName = ''
            kVR = ''

        # Add in default or override whiteboard 
        if kwargs["whiteboard"]:
            whiteboard = kwargs.pop("whiteboard")
            kwargs["whiteboard"] = "%s %s %s" % (self.rNVR, kwargs['type'], whiteboard)
        else:
            kwargs["whiteboard"] = "%s %s" % (self.rNVR, kwargs['type'])

        if kdir and not rKcov:
            sys.stderr.write("Must set --kcov first before kdir\n")
            sys.exit(1)

        # Required kernelinstall test variables.
        kernelparams = []
        kernelparams.append("KERNELARGNAME=kernel")
        kernelparams.append("KERNELARGVERSION=%s" % (kVR))

        if kName == 'kernel-rt':
            arches = ['i386', 'x86_64']
            matrix=getattr(Matrixes, self.getFamily(**kwargs), None).rt
        elif rLite == True:
            matrix=getattr(Matrixes, self.getFamily(**kwargs), None).lite
        elif rVirt == True:
            matrix=getattr(Matrixes, self.getFamily(**kwargs), None).virt
            if family in ['RedHatEnterpriseLinux6', 'RedHatEnterpriseLinux7']:
                arches = ['x86_64']
            else:
                arches = ['i386', 'x86_64', 'ia64']
        else:
            matrix=getattr(Matrixes, self.getFamily(**kwargs), None).matrix
            
        recipeTemplate = BeakerRecipe()
        guestRecipeTemplate = BeakerGuestRecipe()

        # Remove repos before calling addBaseRequires, we only want 
        # them in %post, not during install.
        repos = kwargs.pop("repo", None)
        # Add Distro Requirements
        recipeTemplate.addBaseRequires(*args, **kwargs)
        guestRecipeTemplate.addBaseRequires(*args, **kwargs)

        if repos:
            ks_appends = recipeTemplate.doc.createElement('ks_appends')
            ks_appends_guest = guestRecipeTemplate.doc.createElement('ks_appends')
            for i, repo in enumerate(repos):
                kernel_repo_cmd = '''
cat << EOF >/etc/yum.repos.d/beaker-kernel%(i)s.repo
[beaker-kernel%(i)s]
name=beaker-kernel%(i)s
baseurl=%(repo)s
enabled=1
gpgcheck=0
skip_if_unavailable=1
EOF
''' % dict(i=i, repo=repo)

                # RHEL7 anaconda requires this to be enclosed in %post/%end
                # RHEL6 seems to support both ways
                if self.getFamily(**kwargs) == 'RedHatEnterpriseLinux7':
                    kernel_repo_cmd = '\n%post' + kernel_repo_cmd + '%end\n'

                ks_append = recipeTemplate.doc.createElement('ks_append')
                ks_append.appendChild(recipeTemplate.doc.createCDATASection(kernel_repo_cmd))
                ks_appends.appendChild(ks_append)

                ks_append_guest = guestRecipeTemplate.doc.createElement('ks_append')
                ks_append_guest.appendChild(guestRecipeTemplate.doc.createCDATASection(kernel_repo_cmd))
                ks_appends_guest.appendChild(ks_append_guest)
            recipeTemplate.node.appendChild(ks_appends)
            guestRecipeTemplate.node.appendChild(ks_appends_guest)

        if rKcov:
            myrepo = recipeTemplate.doc.createElement('repo')
            myrepo.setAttribute('name', 'myrepo_%d' % (len(recipeTemplate.repos.childNodes)+1))
            myrepo.setAttribute('url', '%s' % "http://download.devel.redhat.com/qa/rhts/lookaside/lcov/")
            recipeTemplate.addRepo(myrepo)

        recipeTemplate.addTask(rDistInstall)

        # Add system-data test to the start of the AllTests for kernel-rt.
        if (kName == 'kernel-rt'):
            recipeTemplate.addTask("/distribution/pkginstall", taskParams=["PKGARGNAME=rtctl"])
            recipeTemplate.addTask("/distribution/pkginstall", taskParams=["PKGARGNAME=rteval"])
            recipeTemplate.addTask("/kernel/rt-tests/system-data")

        guestRecipeTemplate.addTask(rDistInstall)


        # Added in here since we do not use prcessTemplate
        if rpkginstall:
            paramnode = self.doc.createElement('param')
            paramnode.setAttribute('name' , 'PKGARGNAME')
            paramnode.setAttribute('value' , ' '.join(rpkginstall))

            gpkgParams = copy.copy(paramnode)

            recipeTemplate.addTask('/distribution/pkginstall', paramNodes=[paramnode])
            guestRecipeTemplate.addTask('/distribution/pkginstall', paramNodes=[gpkgParams])

        if r4u9:
            recipeTemplate.addTask("/distribution/systemUpdate", 
                                   taskParams=["addURL=http://download.lab.bos.redhat.com/redhat/released/RHEL-4/U9/AS/$basearch/repo-AS-$basearch/"]) 

        # Create Job
        job = BeakerJob(*args, **kwargs)

        for arch in arches:
            if not arch in matrix:
                sys.stderr.write("skipping arch '%s', no such arch in matrix\n" % (arch))
                continue

            if matrix[arch]:
                recipe_id = randint(1, len(matrix[arch]))
            else:
                recipe_id = None

            for i, matrixArch in enumerate(matrix[arch]):
                myParams = copy.copy(kernelparams)
                myParams.append("KERNELARGVARIANT=%s" %(matrixArch['ktype']))
                recipe = recipeTemplate.clone()

                if self.rNVR:
                    recipe.addTask(rKernInstall, taskParams=myParams)

                if matrixArch.get('fips', False):
                    if without_fips:
                        continue
                    else:
                        recipe.addTask('/distribution/fips/setup-fips-enabled')

                recipe.addAutopick(True)

                if ndump:
                    recipe.addTask('/kernel/networking/ndnc')
                if kdump:
                    recipe.addTask('/kernel/networking/kdump')

                recipe.addDistroRequires("""<distroRequires>
                                             <distro_arch op='=' value='%s'/>
                                            </distroRequires>""" % arch)
                recipe.addHostRequires(matrixArch['sprop'])
                if matrixArch['gprop'] == True and (matrixArch['ktype'] == 'xen' or rKVM):
                    if matrixArch['ktype'] == 'xen':
                        guest1_arch = matrixArch.get('garch', arch)
                        guest2_arch = matrixArch.get('garch', arch)
                    elif arch in ['x86_64']:
                        guest1_arch = matrixArch.get('garch', 'i386')
                        guest2_arch = matrixArch.get('garch', 'x86_64')

                    guestRecipe1 = guestRecipeTemplate.clone()
                    guestRecipe1.addDistroRequires("""<distroRequires>
                                                       <distro_arch op='=' value='%s'/>
                                                      </distroRequires>""" % guest1_arch)
                    guestRecipe2 = guestRecipeTemplate.clone()
                    guestRecipe2.addDistroRequires("""<distroRequires>
                                                       <distro_arch op='=' value='%s'/>
                                                      </distroRequires>""" % guest2_arch)
                    myGuestParams1 = copy.copy(kernelparams)
                    myGuestParams1.append("KERNELARGVARIANT=%s" %(matrixArch['ktype']))
                    myGuestParams2 = copy.copy(kernelparams)
                    myGuestParams2.append("KERNELARGVARIANT=up")
                    if self.rNVR:
                        guestRecipe1.addTask(rKernInstall, taskParams=myGuestParams1)
                        guestRecipe2.addTask(rKernInstall, taskParams=myGuestParams2)

                    self.addTasks(guestRecipe1, taskParams, guest1_arch)
                    self.addTasks(guestRecipe2, taskParams, guest2_arch)

                    if arch == 'ia64':
                        pkgparms = []
                        pkgparms.append("PKGARGNAME=xen-ia64-guest-firmware")
                        recipe.addTask("/distribution/pkginstall", taskParams=pkgparms)
                    recipe.addTask(rVirtInstall)
                    recipe.addTask(rVirtStart)
                    if matrixArch['ktype'] == 'xen':
                        mypackages = self.doc.createElement('packages')
                        mypackage = self.doc.createElement('package')
                        mypackage.setAttribute('name', 'kernel-%s' % (matrixArch['ktype']))
                        mypackages.appendChild(mypackage)
                        recipe.whiteboard = "%s %s DOM0" % (kVR, matrixArch['ktype'])
                        guestRecipe1.node.appendChild(mypackages)
                        guestRecipe1.whiteboard = "%s %s DOMU-PARA" % (guest1_arch,kVR)
                        guestRecipe2.whiteboard = "%s %s DOMU-HVM" % (guest2_arch,kVR)
                        guestRecipe1.guestargs = "--ram=1024 --vcpus=1 --file-size=20 --paravirt --nonsparse"
                        guestRecipe2.guestargs = "--ram=2048 --vcpus=2 --file-size=20 --hvm --nonsparse"
                        guestRecipe1.guestname="%s_xen_pv" % (guest1_arch)
                        guestRecipe2.guestname="%s_xen_hvm" % (guest2_arch)

                    elif matrixArch['gprop'] == True and matrixArch['ktype'] != 'xen':
                        recipe.whiteboard = "%s" % (kVR)
                        guestRecipe1.whiteboard = "%s KVM %s 1 vcpu" % (kVR, guest1_arch)
                        guestRecipe2.whiteboard = "%s KVM %s 2 vcpus" % (kVR, guest2_arch)
                        guestRecipe1.guestargs = "--ram=1024 --vcpus=1 --file-size=20 --hvm --kvm --nonsparse"
                        guestRecipe2.guestargs = "--ram=2048 --vcpus=2 --file-size=20 --hvm --kvm --nonsparse"
                        guestRecipe1.guestname="%s_kvm" % (guest1_arch)
                        guestRecipe2.guestname="%s_kvm" % (guest2_arch)
                        if guestRecipe1.guestname == guestRecipe2.guestname:
                            guestRecipe1.guestname = guestRecipe1.guestname + '_1cpu'

                    recipe.addGuestRecipe(guestRecipe1)
                    recipe.addGuestRecipe(guestRecipe2)
                else:
                    if rAddOn:
                        if i == recipe_id:
                            pkgparms1 = []
                            pkgparms1.append("PHASE=setup")
                            recipe.addTask("/kernel/storage/iscsi-equallogic", taskParams=pkgparms1)

                            pkgparms2 = []
                            pkgparms2.append("MOUNT_DIR=/mnt/iscsi")
                            pkgparms2.append("ARGS=log=log limit=100m bs=128k procs=20")
                            recipe.addTask("/kernel/storage/dt", taskParams=pkgparms2)

                            pkgparms3 = []
                            pkgparms3.append("PHASE=teardown")
                            recipe.addTask("/kernel/storage/iscsi-equallogic", taskParams=pkgparms3)

                    if rKcov:
                        recipe.addTask("/distribution/pkginstall", taskParams=["PKGARGNAME=lcov"])
                        kcovParam = ["MODE=KA"]
                        if kdir is not None:
                            kcovParam.append("KDIR=%s" % kdir)
                        recipe.addTask("/kernel/kcov/prepare", taskParams=kcovParam)
                        recipe.addTask("/kernel/kcov/start", taskParams=[])
                    self.addTasks(recipe, taskParams, arch)
                    if rKcov:
                        recipe.addTask("/kernel/kcov/end", taskParams=[])
                        recipe.addTask("/kernel/kcov/finalize", taskParams=[])
                    if (matrixArch['ktype'] == 'up'):
                        recipe.whiteboard = "%s" % (kVR)
                    else:
                        recipe.whiteboard = "%s %s" % (kVR, matrixArch['ktype'])
                    if matrixArch.get('fips', False):
                        recipe.whiteboard = recipe.whiteboard + ' fips'

                job.addRecipe(recipe)
        
        # jobxml
        jobxml = job.toxml(**kwargs)

        if debug:
            print jobxml

        self.set_hub(username, password)
        submitted_jobs = []
        failed = False

        if not dryrun:
            try:
                submitted_jobs.append(self.hub.jobs.upload(jobxml))
            except Exception, ex:
                failed = True
                print ex

        if not dryrun:
            print "Submitted: %s" % submitted_jobs

        if failed:
            sys.exit(1)

