from bkr.client.task_watcher import *
from bkr.client import BeakerCommand

import sys
import xmlrpclib

class Workflow_Remote(BeakerCommand):
    """Remote workflow to generate job to scheduler"""
    """
    API for Workflow_Remote

    Config
    ------
    COMPONENT is used to look up remote workflow server.
     For Example:
      unless --server is used to specify the remote workflow server we 
      look in the file specified from os.environ["BEAKER_CLIENT_CONF"].
      So if COMPONENT is kernel we look for WORKFLOW_REMOTE_KERNEL.
      The Value must be a URL to an xmlrpc server.
      The component is automatically upper cased.

    Parameters
    ----------
    request: {"workflow_args": []}
          workflow_args: Args to be passed to the remote workflow. This is everything to the right
          of COMPONENT

    Returns
    -------
    remote: {"jobxml": "", "stdout": "", "stderr": "", "retcode": 0}
        jobxml: If defined will contain a valid job xml.
        stdout: If defined will contain stdout from remote workflow
        stderr: If defined will contain stderr from remote workflow
        retcode: return code from remote workflow. Must be provided!
    """

    enabled = True

    def options(self):
        super(Workflow_Remote, self).options()
        self.parser.usage = "%%prog %s [options] COMPONENT [remote_options]" % self.normalized_name
        self.parser.add_option(
            "--dryrun",
            default=False,
            action="store_true",
            help="Don't submit job to scheduler",
        )
        self.parser.add_option(
            "--debug",
            default=False,
            action="store_true",
            help="print the jobxml that it would submit",
        )
        self.parser.add_option(
            "--wait",
            default=False,
            action="store_true",
            help="wait on job completion",
        )
        self.parser.add_option(
            "--server",
            help="Specify Workflow Server, overrides WORKFLOW_SERVER_$COMPONENT from config"
        )
        self.parser.disable_interspersed_args()

    def get_remote_side(self, workflow_name, server=None):
        workflow_server = server or self.conf.get("WORKFLOW_REMOTE_%s" % workflow_name.upper())
        if not workflow_server:
            self.parser.error("No Workflow Server Defined for %s, define in config or override with --server" % workflow_name)
        return workflow_server

    def run(self, *args, **kwargs):
        username = kwargs.get("username", None)
        password = kwargs.get("password", None)

        if len(args) < 1:
            self.parser.error("No COMPONENT specified!")
        workflow_name = args[0]
        remote_args = args[1:]

        remote_url = self.get_remote_side(workflow_name, kwargs.get("server"))
        s = xmlrpclib.ServerProxy(remote_url)
        request={"workflow_args" : remote_args}
        remote = s.run_workflow(workflow_name, request)

        is_failed = int(remote.get("ret_code"))
        if remote.get("stdout"):
            sys.stdout.write(remote.get("stdout") + "\n")
        if remote.get("stderr"):
            sys.stderr.write(remote.get("stderr") + "\n")

        if kwargs.get("debug") and remote.get("jobxml"):
            sys.stdout.write(remote.get("jobxml") + "\n")

        if not kwargs.get("dryrun") and remote.get("jobxml"):
            try:
                self.set_hub(username, password)
                submitted_job = [self.hub.jobs.upload(remote.get("jobxml"))]
                sys.stdout.write("Submitted: %s\n" % submitted_job)
            except (KeyboardInterrupt, SystemExit):
                raise
            except Exception, ex:
                is_failed = True
                sys.stderr.write("Exception: %s\n" % ex)
            if kwargs.get("wait"):
                is_failed |= watch_tasks(self.hub, submitted_job)
        sys.exit(is_failed)

####################################### MAIN ########################################
# The below should never be called in the normal case.  But this is very handy for debugging
# purposes.  
#
# python cmd_workflow_remote.py workflow-remote --help
#
if __name__ == "__main__":
    import bkr.client.main
    class PresetAuthCommandContainer(bkr.client.main.BeakerCommandContainer):
        def __init__(self, conf, **kwargs):
            super(PresetAuthCommandContainer, self).__init__(conf, **kwargs)

    PresetAuthCommandContainer.register_plugin(Workflow_Remote)
    bkr.client.main.BeakerCommandContainer = PresetAuthCommandContainer

    g = globals()
    l = {}

    bkr = "/usr/bin/bkr"
    newargv = []
    newargv.append(bkr)
    newargv.extend(sys.argv[1:])

    sys.argv=newargv
    execfile("/usr/bin/bkr", g, l)
